@@ -31,6 +31,7 @@
 ;; TODOs
 ;; 1) add renamer
 ;; 2) think about cnvt-fctn that generates C code
+;; 3) add code for bytestructures' bounding-struct-descriptor
 
 ;; Issue:
 ;; So issue is when 'typedef struct ref foo_t' has no 'struct def'
@@ -88,7 +89,7 @@
   #:use-module (ice-9 regex)
   #:use-module (ice-9 pretty-print)
   #:re-export (*nyacc-version*)
-  #:version (0 90 2))
+  #:version (0 91 0))
 
 (define fh-cpp-defs
   (cond
@@ -103,44 +104,28 @@
   (append
    `(,(assq-ref %guile-build-info 'includedir) "/usr/include")
    (get-gcc-inc-dirs)))
-(define fh-inc-help
-  (cond
-   ((string-contains %host-type "darwin")
-    '(("__builtin"
-       "__builtin_va_list=void*"
-       "__inline=" "__inline__="
-       "__asm(X)=" "__asm__(X)="
-       "__has_include(X)=__has_include__(X)"
-       "__extension__=" "__restrict=" "__THROW="
-       "__signed=signed"
-       )))
-   (else
-    '(("__builtin"
-       "__builtin_va_list=void*"
-       "__inline=" "__inline__="
-       "__asm(X)=" "__asm__(X)="
-       "__has_include(X)=__has_include__(X)"
-       "__extension__=" "__restrict=" "__THROW="
-       )))))
+
+(define fh-inc-help c99-def-help)
 
 ;; DEBUGGING
 (set! fh-inc-dirs (cons "." fh-inc-dirs))
 
 ;; maybe change to a record-type
 (define *options* (make-parameter '()))
-(define *prefix* (make-parameter ""))	 ; name prefix (e.g., prefix-syms)
-(define *debug* (make-parameter #f))	 ; parse debug mode
-(define *mport* (make-parameter #t))	 ; output module port
-(define *udict* (make-parameter '()))	 ; udecl dict
-(define *ddict* (make-parameter '()))	 ; cpp-def dict
-(define *tdefs* (make-parameter '()))	 ; typenames
-(define *wrapped* (make-parameter '()))	 ; wrappers for foo_t and foo_t*
-(define *defined* (make-parameter '()))	 ; has wrapper and is bytestructure?
-(define *renamer* (make-parameter identity)) ; renamer from ffi-module
-(define *errmsgs* (make-parameter '()))	     ; list of warnings
-;; what about option to trace
+(define *debug-parse* (make-parameter #f)) ; parse debug mode
 (define *echo-decls* (make-parameter #f)) ; add echo-decls code for debugging
 
+(define *prefix* (make-parameter "")) ; name prefix (e.g., prefix-syms)
+(define *renamer* (make-parameter identity)) ; renamer from ffi-module
+
+(define *mport* (make-parameter #t))	   ; output module port
+(define *udict* (make-parameter '()))	   ; udecl dict
+(define *ddict* (make-parameter '()))	   ; cpp-def dict
+(define *tdefs* (make-parameter '()))	   ; typenames
+(define *wrapped* (make-parameter '()))	; wrappers for foo_t and foo_t*
+(define *defined* (make-parameter '()))	; type defined
+
+(define *errmsgs* (make-parameter '()))	; list of warnings
 
 (define (sfscm fmt . args)
   (apply simple-format (*mport*) fmt args))
@@ -287,7 +272,7 @@
 (define (w/struct name) (cons 'struct name))
 (define (w/union name) (cons 'union name))
 (define (w/enum name) (cons 'enum name))
-;; pointers needed?
+;; pointers needed
 (define (w/* name) (cons 'pointer name))
 (define (w/struct* name) (cons 'pointer (cons 'struct name)))
 (define (w/union* name) (cons 'pointer (cons 'union name)))
@@ -301,6 +286,9 @@
 	   seed)))
    '() udict))
 
+(define (rename name)
+  ((*renamer*) name))
+
 ;; === output scheme module header 
 
 (define (ffimod-header path module-opts)
@@ -393,8 +381,9 @@
     (pmatch mdecl-tail
       ;; expand typeref, use renamer,
       (((typename ,name))
-       (or (assoc-ref bs-typemap name)
-	   (string->symbol (string-append name "-desc"))))
+       (let ((name (rename name)))
+	 (or (assoc-ref bs-typemap name)
+	     (string->symbol (string-append name "-desc")))))
 
       (((void)) ''void)
       (((fixed-type "char")) 'int)
@@ -422,15 +411,16 @@
 
       ;; typename use renamers, ... ???
       (((pointer-to) (typename ,name))
-       (cond
-	((assoc-ref bs-typemap name) =>
-	 (lambda (n) `(bs:pointer ,n)))
-	((member (w/* name) defined)
-	 (strings->symbol name "*-desc"))
-	((member name defined)
-	 `(bs:pointer ,(strings->symbol name "-desc")))
-	(else
-	 (strings->symbol name "*-desc"))))
+       (let ((name (rename name)))
+	 (cond
+	  ((assoc-ref bs-typemap name) =>
+	   (lambda (n) `(bs:pointer ,n)))
+	  ((member (w/* name) defined)
+	   (strings->symbol name "*-desc"))
+	  ((member name defined)
+	   `(bs:pointer ,(strings->symbol name "-desc")))
+	  (else
+	   (strings->symbol name "*-desc")))))
 
       (((pointer-to) (void))
        `(bs:pointer 'void))
@@ -482,7 +472,8 @@
 ;; --- output routines ---------------
 
 (define (fhscm-export-def name)
-  (let* ((st-name (if (string? name) name (symbol->string name)))
+  (let* ((name (rename name))
+	 (st-name (if (string? name) name (symbol->string name)))
 	 (sy-name (if (string? name) (string->symbol name) name))
 	 (pred (string->symbol (string-append st-name "?")))
 	 (make (string->symbol (string-append "make-" st-name))))
@@ -493,23 +484,28 @@
 ;;  (sfscm "(export ~A* ~A*? make-~A*)\n" name name name))
 
 (define (fhscm-def-alias name orig)
-  (let ((s-name (if (string? name) (string->symbol name) name))
-	(s-orig (if (string? orig) (string->symbol orig) orig)))
+  (let* ((name (rename name))
+	 (s-name (if (string? name) (string->symbol name) name))
+	 (s-orig (if (string? orig) (string->symbol orig) orig)))
     (ppscm `(define-public ,s-name ,s-orig))))
 
 (define (fhscm-def-desc name desc)
-  (let ((s-name (if (string? name) (string->symbol name) name))
-	(s-desc (if (string? desc) (string->symbol desc) desc)))
+  (let* ((name (rename name))
+	 (s-name (if (string? name) (string->symbol name) name))
+	 (s-desc (if (string? desc) (string->symbol desc) desc)))
     (ppscm `(define-public ,s-name ,s-desc))))
 
 (define (fhscm-def-*desc name)
-  (sfscm "(define-public ~A* (bs:pointer ~A-desc))\n" name name))
+  (let ((name (rename name)))
+    (sfscm "(define-public ~A* (bs:pointer ~A-desc))\n" name name)))
 
 (define (fhscm-def-*desc/delay name)
-  (sfscm "(define-public ~A* (bs:pointer (delay ~A-desc)))\n" name name))
+  (let ((name (rename name)))
+    (sfscm "(define-public ~A* (bs:pointer (delay ~A-desc)))\n" name name)))
 
 (define (fhscm-def-compound name)
-  (let* ((st-name (if (string? name) name (symbol->string name)))
+  (let* ((name (rename name))
+	 (st-name (if (string? name) name (symbol->string name)))
 	 (sy-name (if (string? name) (string->symbol name) name))
 	 (desc (string->symbol (string-append st-name "-desc")))
 	 (pred (string->symbol (string-append st-name "?")))
@@ -518,7 +514,8 @@
     (fhscm-export-def name)))
 
 (define (fhscm-def-pointer name)
-  (let* ((st-name (if (string? name) name (symbol->string name)))
+  (let* ((name (rename name))
+	 (st-name (if (string? name) name (symbol->string name)))
 	 (sy-name (if (string? name) (string->symbol name) name))
 	 (desc (string->symbol (string-append st-name "-desc")))
 	 (pred (string->symbol (string-append st-name "?")))
@@ -527,12 +524,14 @@
     (fhscm-export-def name)))
 
 (define (fhscm-def-pointer/delay name)
-  (sfscm "(define-fh-pointer-type ~A* ~A*-desc\n" name)
-  (sfscm "  ~A*? make-~A*)\n" name name)
-  (fhscm-export-def name))
+  (let ((name (rename name)))
+    (sfscm "(define-fh-pointer-type ~A* ~A*-desc\n" name)
+    (sfscm "  ~A*? make-~A*)\n" name name)
+    (fhscm-export-def name)))
 
 (define (fhscm-ref-deref typename)
-  (let* ((type* (strings->symbol typename "*"))
+  (let* ((typename (rename typename))
+	 (type* (strings->symbol typename "*"))
 	 (make* (strings->symbol "make-" typename "*"))
 	 (type (strings->symbol typename))
 	 (make (strings->symbol "make-" typename)))
@@ -564,6 +563,9 @@
 
 ;; --- structs and unions
 
+;; scheme-bytestructures will be adding
+;;   bounding-struct-descriptor union-desc => struct-desc
+
 ;; This routine will munge the fields and then perform typeref expansion.
 ;; `defined' here means has -desc (what?)
 (define (expand-field-list-typerefs field-list)
@@ -621,11 +623,16 @@
 	 (ty*-desc (and typename (strings->symbol typename "*-desc")))
 	 (ag-desc (and aggrname (strings->symbol aggrname "-desc")))
 	 (ag*-desc (and aggrname (strings->symbol aggrname "*-desc")))
-	 (packed? (and=> (assoc-ref attr 'attributes)
+	 (cattr (assoc-ref attr 'attributes)) ;; __attributes__
+	 (packed? (and=> cattr
 			 (lambda (l) (string-contains (car l) "__packed__" 0))))
+	 (aligned? (and=> cattr
+			 (lambda (l) (string-contains (car l) "__alignof__" 0))))
 	 (bs-spec (if packed?
 		      (list bs-aggr-t #t `(list ,@sflds))
 		      (list bs-aggr-t `(list ,@sflds)))))
+    (if aligned? (sferr "ffi-help: not processing __aligned__ in ~S\n"
+			(or aggr-t typename)))
     (cond
      ((and typename aggr-name)
       ;;(sfscm ";; == ~A =>\n" typename)
@@ -778,6 +785,7 @@
     (((union-def (field-list . ,fields)))
      ;; TODO check libffi on how unions are passed and returned.
      ;; I assume here never passed as a floating point.
+     ;; This should use bounding-struct-descriptor from bytestructures
      (let iter ((type #f) (size 0) (flds fields))
        (if (null? flds)
 	   (case type ((double) 'ffi:uint64) ((float) 'ffi:uint32) (else type))
@@ -1328,10 +1336,16 @@
 	 (stor-spec (typedef))
 	 (type-spec (struct-def (@ . ,attr1) (ident ,struct-name) ,field-list)))
 	(init-declr (ptr-declr (pointer) (ident ,typename))))
-       (cnvt-struct-def attr1 (sw/* typename) struct-name field-list)
+       (cnvt-struct-def attr1 #f struct-name field-list)
+       (sfscm "(define-public ~A-desc struct-~A*-desc)\n" typename struct-name)
+       (fhscm-def-pointer typename)
        (values
-	(cons* (w/* typename) (w/struct struct-name) wrapped)
-	(cons* (w/* typename) (w/struct struct-name) defined)))
+	(cons* typename (w/* typename)
+	       (w/struct struct-name) (w/struct* struct-name)
+	       wrapped)
+	(cons* typename (w/* typename)
+	       (w/struct struct-name) (w/struct* struct-name)
+	       defined)))
 
       ;; typedef struct { ... } foo_t;
       ((udecl
@@ -1786,6 +1800,15 @@
 ;; replacemnts down to constants (strings, integers, etc)
 (define (gen-lookup-proc keep-defs cpp-defs ext-mods)
 
+  (define err-port (open-output-file "/dev/null"))
+  
+  (define* (try-parse-repl repl)
+    (with-error-to-port err-port
+      (lambda ()
+	(catch 'c99-error
+	  (lambda () (parse-c99x repl (*tdefs*) #:cpp-defs cpp-defs))
+	  (lambda args #f)))))
+
   ;; @var{keep-defs} is list of CPP defs and enum key/val pairs. It is
   ;; possible for an enum symbol to be used as a macro function so we
   ;; need to first check for integer before trying expand-cpp-macro-ref.
@@ -1812,17 +1835,11 @@
 		((zero? (string-length repl)) seed)
 		;;
 		((cintstr->num repl) => (lambda (val) (acons symb val seed)))
-		((with-error-to-port (open-output-file "/dev/null")
-		   (lambda ()
-		     (catch 'c99-error
-		       (lambda ()
-			 (parse-c99x repl (*tdefs*) #:cpp-defs cpp-defs))
-		       (lambda args #f))))
+		((try-parse-repl repl)
 		 => (lambda (val)
 		      (let ((cv (eval-c99-cx val (*udict*) (*ddict*))))
 			(unless cv
-			  (sferr "ffi-help: unable to generate constant for ~S\n"
-				 name))
+			  (sfscm ";; unable to generate constant for ~S\n" name))
 			(if cv (acons symb cv seed) seed))))
 		;;
 		(else
@@ -1848,7 +1865,8 @@
 	 ((symbol? obj) (,sv-name obj))
 	 ((fh-object? obj) (struct-ref obj 0)) ;; ???
 	 (else (error "type mismatch")))))
-    ))
+    
+    (close err-port)))
 
 
 ;; === Parsing the C header(s)
@@ -1877,7 +1895,7 @@
 		       #:inc-help inc-help
 		       #:mode 'decl
 		       #:show-incs #f
-		       #:debug (*debug*))))
+		       #:debug (*debug-parse*))))
 	(fherr "parse failed"))))
 
 ;; @deffn parse-includes attrs
@@ -1979,6 +1997,10 @@
 (define (expand-ffi-module-spec path module-options)
   (check-deps module-options)
   (let* ((script-options (*options*))
+	 (dbugl (or (and=> (assq-ref script-options 'debug)
+			   (lambda (v) (map string->symbol
+					    (string-split v #\,))))
+		    '()))
 	 (mbase (path->path path))
 	 (dirpath (derive-dirpath (assq-ref script-options 'file) mbase))
 	 (mfile (string-append dirpath mbase ".scm"))
@@ -1989,11 +2011,13 @@
 	 (declf (or (assq-ref attrs 'decl-filter) identity))
 	 (renamer (or (assq-ref attrs 'renamer) identity))
 	 ;;
-	 (tree (cond
-		((assq-ref attrs 'include) (parse-includes attrs))
-		((assq-ref attrs 'api-code) =>
-		 (lambda (code) (parse-code code attrs)))
-		(else (fherr "expecing #:includes or #:api-code"))))
+	 (tree (begin
+		 (if (memq 'parse dbugl) (*debug-parse* #t))
+		 (cond
+		  ((assq-ref attrs 'include) (parse-includes attrs))
+		  ((assq-ref attrs 'api-code) =>
+		   (lambda (code) (parse-code code attrs)))
+		  (else (fherr "expecing #:includes or #:api-code")))))
 	 (udecls (c99-trans-unit->udict tree #:inc-filter incf))
 	 (udict (c99-trans-unit->udict/deep tree))
 	 (ffi-decls (map car udecls))	; just the names, get decls from udict
@@ -2036,9 +2060,9 @@
     (*udict* udict)
     (*mport* mport)
     (*ddict* ddict)
+    (*renamer* renamer)
     (*tdefs* (udict->typenames udict))
-    (if (assq-ref script-options 'debug) (*echo-decls* #t))
-    ;; renamer?
+    (if (memq 'echo-decls dbugl) (*echo-decls* #t))
 
     ;; file and module header
     (ffimod-header path module-options)
@@ -2070,13 +2094,10 @@
     (ugly-print ffimod-defined mport #:per-line-prefix "   " #:trim-ends #t)
     (sfscm ")\n(export ~A-types)\n" (path->name path))
 
-    ;;(sfscm "\n;; TODO: add renamer\n")
-
-    ;; return port so compiler can output remaining code
+    ;; Return the output port so the compiler can output remaining code.
     mport))
 
 
-
 ;; === test compiler ================
 
 ;; @deffn {Procedure} C-fun-decl->scm code
