@@ -15,6 +15,16 @@
 ;; You should have received a copy of the GNU Lesser General Public License
 ;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
+;;; Notes:
+
+;; The syntax of SXML trees is simple:
+;; @example
+;; expr => (tag item @dots{}) | (tag (@@ attr @dots{}) item @dots{})
+;; item => string | expr
+;; attr => (tag . string)
+;; @end example
+
+
 ;;; Code:
 
 (define-module (nyacc lang sx-util)
@@ -23,8 +33,9 @@
 	    sx-has-attr? sx-attr-ref sx-attr-add sx-attr-add* sx-attr-set!
 	    sx-find
 	    sx-split sx-split* sx-join sx-join* sx-cons* sx-list
+	    sx-unitize
 	    sx-match)
-  #:use-module ((srfi srfi-1) #:select (find fold fold-right)))
+  #:use-module ((srfi srfi-1) #:select (find fold fold-right append-reverse)))
 (cond-expand
   (mes)
   (guile-2)
@@ -225,8 +236,8 @@
       (set-cdr! sx (cons `(@ (,key ,val)) (cdr sx))))
   sx)
 
-;; @deffn {Procedure} sx-cons* tag attr ... => sx
-;; @deffnx {Procedure} sx-list tag attr ... => sx
+;; @deffn {Procedure} sx-cons* tag attr exp ... tail => sx
+;; @deffnx {Procedure} sx-list tag attr exp ... => sx
 ;; Generate the tag and the attr list if it exists.  Note that
 ;; The following are equivalent:
 ;; @example
@@ -234,6 +245,8 @@
 ;; (sx-list tag attr elt1 elt2)
 ;; @end example
 ;; @noindent
+;; Expressions that are @code{#f} or @code{'()} will be skipped;
+;; they should be strings or pairs.
 ;; @end deffn
 (define (sx-cons* tag attr . rest)
   (if (null? rest) (error "expecing tail"))
@@ -243,10 +256,11 @@
 	       ((pair? (car attr)) `(@ . ,attr))
 	       (else attr)))
 	(tail (let loop ((items rest))
-		(if (null? (cdr items)) (car items)
-		    (if (car items)
-			(cons (car items) (loop (cdr items)))
-			(loop (cdr items)))))))
+		(cond
+		 ((null? (cdr items)) (car items))
+		 ((not (car items)) (loop (cdr items)))
+		 ((null? (car items)) (loop (cdr items)))
+		 (else (cons (car items) (loop (cdr items))))))))
     (if attr (cons* tag attr tail) (cons tag tail))))
 (define (sx-list tag attr . rest)
   (let ((attr (cond
@@ -255,10 +269,11 @@
 	       ((pair? (car attr)) `(@ . ,attr))
 	       (else attr)))
 	(tail (let loop ((items rest))
-		(if (null? items) '()
-		    (if (car items)
-			(cons (car items) (loop (cdr items)))
-			(loop (cdr items)))))))
+		(cond
+		 ((null? items) '())
+		 ((not (car items)) (loop (cdr items)))
+		 ((null? (car items)) (loop (cdr items)))
+		 (else (cons (car items) (loop (cdr items))))))))
     (if attr (cons* tag attr tail) (cons tag tail))))
 
 ;; @deffn {Procedure} sx-split sexp => tag attr tail
@@ -302,6 +317,23 @@
 	    (cons* tag `(@ ,attr) tail))
 	(cons tag tail))))
 
+;; @deffn {Procedure} sx-unitize list-tag form seed
+;; Given a declaration of form @code{(tag ... (elt-list ...) ...)}
+;; fold into the seed broken list of
+;; @code{(tag ... elt1 ...) (tag ... elt2 ...) ... seed}.
+;; Any attributes for the list form are lost.
+;; @end deffn
+(define (sx-unitize list-tag form seed)
+  (let loop ((head '()) (elts '()) (tail '()) (form form))
+    (if (null? elts)
+	(if (and (pair? (car form)) (eq? list-tag (sx-tag (car form))))
+	    (loop head (cdar form) (cdr form) '())
+	    (loop (cons (car form) head) elts tail (cdr form)))
+	(let loop2 ((elts elts))
+	  (if (null? elts) seed
+	      (cons (append-reverse (cons (car elts) head) tail)
+		    (loop2 (cdr elts))))))))
+
 ;; ============================================================================
 
 ;; sx-match: somewhat like sxml-match but hoping to be more usable and more
@@ -403,6 +435,9 @@
     ((_ tv t0 kt kf)
      (if (eqv? tv 't0) kt kf))))
 
+(define (rule-error s) (error "rule contains non-SXML element" s))
+(define (expr-error v) (error "expression contains not-SXML element" v))
+
 ;; sxm-tail val pat kt kf
 ;; match tail of sexp = list of nodes
 (define-syntax sxm-tail
@@ -425,8 +460,8 @@
     ((_ v (hp . tp) kt kf) (if (pair? v) (sxm-sexp v (hp . tp) kt kf) kf))
     ((_ v s kt kf)
      (begin
-       (unless (string? s) (error "expecting string"))
-       (unless (string? v) (error "expecting string"))
+       (unless (string? s) (rule-error s))
+       (unless (string? v) (expr-error v))
        (if (string=? s v) kt kf)))
     ;;((_ v s kt kf) (if (string=? s v) kt kf))
     ;;^-- If not pair or unquote then must be string, right?
