@@ -30,9 +30,9 @@
   #:use-module ((system base compile) #:select (compile-file))
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-37)
-  #:version (0 90 2))
+  #:version (0 91 0))
 
-(define *ffi-help-version* "0.90.2")
+(define *ffi-help-version* "0.91.0")
 
 (define %summary
   "Compile a ffi-file (C interface spec) to Scheme (or maybe .go).")
@@ -72,7 +72,7 @@ Generate a Guile Scheme file from the source FFI file FILE.
   -L  --load-path=DIR  add DIR to the front of the module load path
   -I  --inc-dir=DIR    add DIR to list of dir's to search for C headers
   -o, --output=OFILE   write output to OFILE
-  -d, --debug=x,y,z    set debug flags (e.g., echo-decl)
+  -d, --debug=x,y      set debug flags: echo-decl, parse
   -X, --no-exec        don't generate .go file(s)
   -R, --no-recurse     don't do recursive compile on dep's
 
@@ -90,7 +90,7 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
 	     (show-version) (exit 0)))
    (option '(#\d "debug") #t #f
 	   (lambda (opt name arg opts files)
-	     (values (acons/seed 'debug arg opts) files)))
+	     (values (acons 'debug arg opts) files)))
    (option '(#\o "output") #t #f
 	   (lambda (opt name arg opts files)
 	     (if (assoc-ref opts 'output-file)
@@ -107,10 +107,10 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
 	     (values (acons 'no-recurse #t opts) files)))
    (option '(#\X "no-exec") #f #f
 	   (lambda (opt name arg opts files)
-	     (values (acons 'no-exec arg opts) files)))
+	     (values (acons 'no-exec #t opts) files)))
    (option '("any-suffix") #f #f
 	   (lambda (opt name arg opts files)
-	     (values (acons 'no-exec arg opts) files)))
+	     (values (acons 'any-sufffix #t opts) files)))
    ))
 
 ;; from scripts/compile.scm
@@ -198,30 +198,33 @@ Report bugs to https://savannah.nongnu.org/projects/nyacc.\n"))
       ((assq-ref options 'no-recurse)
        (warn "please recompile `~A'" dep-file))
       (else
-       (compile-ffi dep-file options))))
+       (compile-ffi dep-file (acons 'no-exec #f options)))))
    (updated-ffi-deps file options)))
 
 ;; -----------------------------------------------------------------------------
 
+(define (cleanup path)
+  (basename path))
+
 (define* (compile-ffi ffi-file options #:key module)
   (let* ((base (string-drop-right ffi-file 4))
 	 (scm-file (string-append base ".scm")))
     (ensure-ffi-deps ffi-file options)
     (catch 'ffi-help-error
       (lambda ()
-	(sfmt "compiling `~A' ..." ffi-file)
+	(sfmt "compiling `~A' ...\n" (cleanup ffi-file))
 	(compile-ffi-file ffi-file options)
-	(sfmt " wrote `~A'\n" scm-file))
+	(sfmt "... wrote `~A'\n" (cleanup scm-file)))
       (lambda (key fmt . args)
 	(apply fail fmt args)
 	(exit 1)))
     (unless (assq-ref options 'no-exec)
-      (sfmt "compiling `~A' ..." scm-file)
+      (sfmt "compiling `~A' ...\n" (cleanup scm-file))
        (let ((go-file (compile-file scm-file
 				   #:from 'scheme #:to 'bytecode
 				   #:opts '())))
 	(load-compiled go-file)
-	(sfmt " wrote `~A'\n" go-file))
+	(sfmt "... wrote `~A'\n" (cleanup go-file)))
       (sleep 1))))
 
 (define (main . args)
